#include <vector>
#include <string>
#include <algorithm>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>
#include <stdarg.h>
#include <ctype.h>
#if defined(_WIN32) || defined(_WIN64)
#define NOMINMAX
#include <windows.h>
#else // _WIN32 || _WIN64
// ...
#endif // _WIN32 || _WIN64

#define SHA1HashSize 20

/*
 *  This structure will hold context information for the SHA-1
 *  hashing operation
 */
typedef struct SHA1Context
{
    uint32_t Intermediate_Hash[SHA1HashSize/4]; /* Message Digest  */

    uint32_t Length_Low;            /* Message length in bits      */
    uint32_t Length_High;           /* Message length in bits      */

                               /* Index into message block array   */
    int_least16_t Message_Block_Index;
    uint8_t Message_Block[64];      /* 512-bit message blocks      */

    int Computed;               /* Is the digest computed?         */
    int Corrupted;             /* Is the message digest corrupted? */
} SHA1Context;

/*
 *  Function Prototypes
 */
int SHA1Reset(SHA1Context*);
int SHA1Input(SHA1Context*, const void*, unsigned int);
int SHA1Result(SHA1Context*, uint8_t Message_Digest[SHA1HashSize]);

void Format(std::string &r_s_result, const char *p_s_fmt, ...) // throw(std::bad_alloc)
{
	va_list ap;

	size_t n_size = strlen(p_s_fmt) + 1;
	r_s_result.resize(n_size);
	// alloc str as long as formatting str (should be enough in some cases)

	for(;;) {
		va_start(ap, p_s_fmt);
#if defined(_MSC_VER) && !defined(__MWERKS__)
#if _MSC_VER >= 1400
		int n = _vsnprintf_s(&r_s_result[0], n_size * sizeof(char), _TRUNCATE, p_s_fmt, ap);
#else // _MSC_VER >= 1400
		int n = _vsnprintf(&r_s_result[0], n_size - 1, p_s_fmt, ap);
		// maximum characters to write, not the size of buffer
#endif // _MSC_VER >= 1400
#else // _MSC_VER && !__MWERKS__
		int n = vsnprintf(&r_s_result[0], n_size * sizeof(char), p_s_fmt, ap);
#endif // _MSC_VER && !__MWERKS__
		va_end(ap);
		// try to sprintf

		if(n >= 0 && unsigned(n) < n_size) {
#if defined(_MSC_VER) && !defined(__MWERKS__)
			r_s_result.resize(n);
#else // _MSC_VER && !__MWERKS__
			r_s_result.resize(strlen(r_s_result.data())); // doesn't need c_str(), terminating zero is already there
#endif // _MSC_VER && !__MWERKS__
			assert(r_s_result.length() == strlen(r_s_result.data()));
			return;
		}
		// see if we made it

		if(n >= 0) // glibc 2.1
			n_size = n + 1; // precisely what is needed
		else // glibc 2.0, msvc
			n_size *= 2;

		r_s_result.resize(n_size);
		// realloc to try again
	}
}

/**
 *	@brief gets temporary directory the user can write to
 *	@param[out] r_s_path is the path to the temporary directory, never ends with a slash
 *	@return Returns true on success, false on failure.
 */
bool Get_TempDirectory(std::string &r_s_path) // throw(std::bad_alloc)
{
#if defined(_WIN32) || defined(_WIN64)
	r_s_path.resize(GetTempPath(0, NULL) + 1);
	if(!GetTempPathA((DWORD)r_s_path.size(), &r_s_path[0])) // size won't exceed DWORD_MAX, since it's read from DWORD
		return false; // something went wrong
	assert(strlen(r_s_path.c_str()) > 0 && r_s_path[strlen(r_s_path.c_str()) - 1] == '\\'); // the returned string ends with a backslash
	r_s_path.resize(strlen(r_s_path.c_str()) - 1); // cut the backslash here
	// get temp path (eg. "c:\\windows\\temp")
#else // _WIN32 || _WIN64
#if 0 // g++ linker warns about using mktemp(), don't want to use that anymore
	char p_s_temp[256] = "/tmp/fileXXXXXX";
	if(!mktemp(p_s_temp)) // do *not* use mkstemp(), do not want the file to be lying around
		return false;
	assert(strrchr(p_s_temp + 1, '/')); // should contain slash
	*(char*)strrchr(p_s_temp + 1, '/') = 0; // erase the last slash (hence the string does not contain it)
	r_s_path = p_s_temp;
	// get temp file name and erase the file name to get the path
#else // 0
	const char *p_s_temp = getenv("TMPDIR"); // environment variable
	// The caller must take care not to modify this string, since that would change the
	// environment of the process. Do not free it either.
	// (e.g. on cluster computers, temp is often directory specific to the job id, like "/tmp/pbs.132048.dm2")

	if(!p_s_temp) {
		if(P_tmpdir)
			p_s_temp = P_tmpdir; // in stdio.h
		else {
			p_s_temp = "/tmp"; // just hope it is there

			/*TFileInfo t_temp_info(p_s_temp);
			if(!t_temp_info.b_exists || !t_temp_info.b_directory)
				return false;*/
			// don't want to depend on Dir.cpp, some apps already use only the header
		}
	}
	// fallbacks if the environment variable is not set

	r_s_path = p_s_temp;
	if(!r_s_path.empty() && r_s_path[r_s_path.length() - 1] == '/')
		r_s_path.erase(r_s_path.end() - 1);
	// get rid of the trailing slash
#endif // 0
#endif // _WIN32 || _WIN64

	/*assert(!b_EndsWithSlash(r_s_path));
	assert(b_Is_Normalized(r_s_path));
	assert(b_Is_Absolute(r_s_path));*/
	// make sure there is no slash at the end, and that the path is normalized

	return true;
}

bool Get_TempFileName(std::string &r_s_temp_file_name, const char *p_s_app_id) // throw(std::bad_alloc)
{
	assert(p_s_app_id && strlen(p_s_app_id));
	// may not be emtpy

#if defined(_WIN32) || defined(_WIN64)
	std::string s_temp_path;
	s_temp_path.resize(GetTempPath(0, NULL) + 1);
	if(!GetTempPathA((DWORD)s_temp_path.size(), &s_temp_path[0])) // size won't exceed DWORD_MAX, since it's read from DWORD
		return false; // something went wrong
	s_temp_path.resize(strlen(s_temp_path.c_str()));
	// get temp path (eg. "c:\windows\temp")

	r_s_temp_file_name.resize(s_temp_path.length() + 16 + strlen(p_s_app_id));
	if(!GetTempFileNameA(s_temp_path.c_str(), p_s_app_id, 0, &r_s_temp_file_name[0]))
		return false; // something went wrong
	r_s_temp_file_name.resize(strlen(r_s_temp_file_name.c_str()));
	// get temp filename
#else // _WIN32 || _WIN64
	std::string s_tmp;
	if(!Get_TempDirectory(s_tmp)) // use "proper" temp (e.g. on cluster computers, temp is often directory specific to the job id, like "/tmp/pbs.132048.dm2")
		return false;
	Format(r_s_temp_file_name, "%s/%sXXXXXX", s_tmp.c_str(), p_s_app_id); // had 8 X's // 2012-07-17 change // t_odo - carry this change to documentation
	// 2013-11-13 changed template to not include the /tmp folder as that might not be the location of tmp
	// make template

	int n_file;
	if((n_file = mkstemp((char*)r_s_temp_file_name.c_str())) < 0)
		return false;
	close(n_file);
	// create temp file
#endif // _WIN32 || _WIN64

	return true;
}

bool ReadFile(std::string &r_s_output, const char *p_s_filename) // throw(std::bad_alloc)
{
	FILE *p_fr;
#if defined(_MSC_VER) && !defined(__MWERKS__) && _MSC_VER >= 1400
	if(fopen_s(&p_fr, p_s_filename, "rb"))
#else //_MSC_VER && !__MWERKS__ && _MSC_VER >= 1400
	if(!(p_fr = fopen(p_s_filename, "rb")))
#endif //_MSC_VER && !__MWERKS__ && _MSC_VER >= 1400
		return false;
	unsigned int n_length;
	if(fseek(p_fr, 0, SEEK_END) ||
	   (n_length = ftell(p_fr)) < 0 ||
	   fseek(p_fr, 0, SEEK_SET)) {
		fclose(p_fr);
		return false;
	}
	try {
		r_s_output.resize(n_length);
	} catch(std::bad_alloc &r_exc) {
		fclose(p_fr);
		throw r_exc; // rethrow
	}
	if(fread(&r_s_output[0], sizeof(char), n_length, p_fr) != n_length) {
		fclose(p_fr);
		return false;
	}
	fclose(p_fr);
	return true;
}

void TrimSpace(std::string &r_s_string)
{
	size_t b = 0, e = r_s_string.length();
	while(e > 0 && isspace(uint8_t(r_s_string[e - 1])))
		-- e;
	while(b < e && isspace(uint8_t(r_s_string[b])))
		++ b;
	r_s_string.erase(e);
	r_s_string.erase(0, b);
}

void Split(std::vector<std::string> &r_s_dest, const std::string &r_s_string,
	const char *p_s_separator, int n_thresh)
{
	r_s_dest.clear();
	const size_t n_separator_skip = strlen(p_s_separator);

	size_t n_pos = 0;
	size_t n_next_pos;
	while((n_next_pos = r_s_string.find(p_s_separator, n_pos)) != std::string::npos) {
		if(n_thresh < 0 || n_next_pos - n_pos > unsigned(n_thresh)) {
			r_s_dest.resize(r_s_dest.size() + 1);
			std::string &r_s_new = r_s_dest.back();
			r_s_new.insert(r_s_new.begin(), r_s_string.begin() + n_pos,
				r_s_string.begin() + n_next_pos);
		}
		n_pos = n_next_pos + n_separator_skip; // skip over the separator
	}
	if(n_thresh < 0 || r_s_string.length() - n_pos > unsigned(n_thresh)) {
		r_s_dest.resize(r_s_dest.size() + 1);
		std::string &r_s_new = r_s_dest.back();
		r_s_new.insert(r_s_new.begin(), r_s_string.begin() + n_pos,
			r_s_string.end());
	}
}

int main(int n_arg_num, const char **p_arg_list)
{
	try {
		if(n_arg_num != 2) {
			fprintf(stderr, "error: use IsTexReady <jobname>\n");
			return -1;
		}
		const char *p_s_jobname = p_arg_list[1];
		if(!strcmp(p_s_jobname, "-h") || !strcmp(p_s_jobname, "--help") ||
		   !strcmp(p_s_jobname, "/?") || !strcmp(p_s_jobname, "--usage")) {
			printf("use: IsTexReady <jobname>\n\n"
				"It will return 0 in case there is no need to re-run tex,\n"
				"1 in case tex should be run once more or -1 on error.\n\n"
				"Run in the same folder where <jobname>.log can be found.\n");
			return 0; // not sure
		}
		// get jobname

		std::string s_logfile_name = std::string(p_s_jobname) + ".log";

		bool b_rerun = false;

		std::string s_logfile;
		if(!ReadFile(s_logfile, s_logfile_name.c_str())) {
			fprintf(stderr, "error: while reading \'%s\'\n", s_logfile_name.c_str());
			b_rerun = true; // should re-run as the log file might be simply missing
		}
		// read logfile

		if(s_logfile.find("Rerun to get") != std::string::npos) {
			printf("IsTexReady: should rerun latex as per the logfile\n");
			b_rerun = true;
		}
		// have "Rerun to get" something, should run again

		std::vector<std::string> aux_list;
		{
			std::string s_aux_list_name;
			if(!Get_TempFileName(s_aux_list_name, "trdy"))
				throw std::runtime_error("Get_TempFileName() failed");
			// get a temp file

#if defined(_WIN32) || defined(_WIN64)
			system(("dir /B /S *.aux *.toc *.lof *.lot *.loa *.lol *.idx > " + s_aux_list_name).c_str());
#else // _WIN32 || _WIN64
			system(("find . -name \'*.aux\' -or -name \'*.toc\' -or -name \'*.lof\' "
				"-or -name \'*.lot\' -or -name \'*.loa\' -or -name \'*.lol\' -or -name \'*.idx\' > " + s_aux_list_name).c_str());
#endif // _WIN32 || _WIN64
			// list the files in it (aux, toc, list of figures / tables / algorithms / listings, idx)

			std::string s_aux_list;
			if(!ReadFile(s_aux_list, s_aux_list_name.c_str()))
				throw std::runtime_error("getting a list of aux files failed");
			// read it back

			Split(aux_list, s_aux_list, "\n", 0); // split the list by newlines
			std::for_each(aux_list.begin(), aux_list.end(), TrimSpace); // remove leading or trailing spaces
			std::sort(aux_list.begin(), aux_list.end()); // sort the filenames
			aux_list.erase(std::unique(aux_list.begin(), aux_list.end()), aux_list.end()); // remove duplicates (empty entries, whitespace, ...)
			std::vector<std::string>::iterator p_empty_it;
			if((p_empty_it = std::find(aux_list.begin(), aux_list.end(), std::string())) != aux_list.end())
				aux_list.erase(p_empty_it); // erase any empty entries
			// split to the individual entries

			remove(s_aux_list_name.c_str());
			// remove the temp file
		}
		// get a list of all the aux files

		std::string s_hash_string;
		{
			SHA1Context sha1;
			SHA1Reset(&sha1);
			for(size_t i = 0, n = aux_list.size(); i < n; ++ i) {
				SHA1Input(&sha1, aux_list[i].c_str(), aux_list[i].size() * sizeof(char));
				// hash the file name

				std::string s_aux_file;
				if(!ReadFile(s_aux_file, aux_list[i].c_str()))
					throw std::runtime_error("failed to read one or more aux file(s)");
				SHA1Input(&sha1, s_aux_file.c_str(), s_aux_file.size() * sizeof(char));
				// hash the file contents
			}
			uint8_t Message_Digest[SHA1HashSize];
			SHA1Result(&sha1, Message_Digest);
			for(int i = 0; i < SHA1HashSize; ++ i) {
				std::string s_hash_digit;
				Format(s_hash_digit, "%02x", Message_Digest[i]);
				s_hash_string += s_hash_digit;
			}
		}
		// get SHA1 of the concatenated files

		std::string s_hashes_name = std::string(p_s_jobname) + "_aux.sha1";

		std::string s_hashes;
		if(!b_rerun && !ReadFile(s_hashes, s_hashes_name.c_str())) {
			printf("IsTexReady: should rerun latex as per missing aux file hashes\n");
			b_rerun = true;
		}
		// read aux file hashes. if not found, should run again

		TrimSpace(s_hashes);
		if(!b_rerun && s_hashes != s_hash_string) {
			printf("IsTexReady: should rerun latex as the aux files have changed\n");
			b_rerun = true;
		}

		if(b_rerun) {
			FILE *p_fw;
#if defined(_MSC_VER) && !defined(__MWERKS__) && _MSC_VER >= 1400
			if(fopen_s(&p_fw, s_hashes_name.c_str(), "w"))
#else //_MSC_VER && !__MWERKS__ && _MSC_VER >= 1400
			if(!(p_fw = fopen(s_hashes_name.c_str(), "w")))
#endif //_MSC_VER && !__MWERKS__ && _MSC_VER >= 1400
				throw std::runtime_error("failed to write the digest of the aux files");
			fprintf(p_fw, "%s\n", s_hash_string.c_str());
			fclose(p_fw);
			// save a new hashes file

			return 1;
			// signal to rerun
		}
		// should we rerun?

		printf("IsTexReady: no need to run latex again\n");
	} catch(std::bad_alloc&) {
		fprintf(stderr, "error: uncaught std::bad_alloc\n");
		return -1;
	} catch(std::runtime_error &r_exc) {
		fprintf(stderr, "error: uncaught std::runtime_error: \'%s\'\n", r_exc.what());
		return -1;
	} catch(std::exception&) {
		fprintf(stderr, "error: uncaught std::exception\n");
		return -1;
	}

	return 0; // noneed to run again, all is ready.
}

// ------------------------------------------------------------------------------------------------

// SHA-1 due to https://tools.ietf.org/html/rfc3174

#ifndef _SHA_enum_
#define _SHA_enum_
enum
{
    shaSuccess = 0,
    shaNull,            /* Null pointer parameter */
    shaInputTooLong,    /* input data too long */
    shaStateError       /* called Input after Result */
};
#endif

/*
 *  sha1.c
 *
 *  Description:
 *      This file implements the Secure Hashing Algorithm 1 as
 *      defined in FIPS PUB 180-1 published April 17, 1995.
 *
 *      The SHA-1, produces a 160-bit message digest for a given
 *      data stream.  It should take about 2**n steps to find a
 *      message with the same digest as a given message and
 *      2**(n/2) to find any two messages with the same digest,
 *      when n is the digest size in bits.  Therefore, this
 *      algorithm can serve as a means of providing a
 *      "fingerprint" for a message.
 *
 *  Portability Issues:
 *      SHA-1 is defined in terms of 32-bit "words".  This code
 *      uses <stdint.h> (included via "sha1.h" to define 32 and 8
 *      bit unsigned integer types.  If your C compiler does not
 *      support 32 bit unsigned integers, this code is not
 *      appropriate.
 *
 *  Caveats:
 *      SHA-1 is designed to work with messages less than 2^64 bits
 *      long.  Although SHA-1 allows a message digest to be generated
 *      for messages of any number of bits less than 2^64, this
 *      implementation only works with messages with a length that is
 *      a multiple of the size of an 8-bit character.
 *
 */

/*
 *  Define the SHA1 circular left shift macro
 */
#define SHA1CircularShift(bits,word) \
                (((word) << (bits)) | ((word) >> (32-(bits))))

/* Local Function Prototyptes */
void SHA1PadMessage(SHA1Context *);
void SHA1ProcessMessageBlock(SHA1Context *);

/*
 *  SHA1Reset
 *
 *  Description:
 *      This function will initialize the SHA1Context in preparation
 *      for computing a new SHA1 message digest.
 *
 *  Parameters:
 *      context: [in/out]
 *          The context to reset.
 *
 *  Returns:
 *      sha Error Code.
 *
 */
int SHA1Reset(SHA1Context *context)
{
    if (!context)
    {
        return shaNull;
    }

    context->Length_Low             = 0;
    context->Length_High            = 0;
    context->Message_Block_Index    = 0;

    context->Intermediate_Hash[0]   = 0x67452301;
    context->Intermediate_Hash[1]   = 0xEFCDAB89;
    context->Intermediate_Hash[2]   = 0x98BADCFE;
    context->Intermediate_Hash[3]   = 0x10325476;
    context->Intermediate_Hash[4]   = 0xC3D2E1F0;

    context->Computed   = 0;
    context->Corrupted  = 0;
	 return shaSuccess;
}

/*
 *  SHA1Result
 *
 *  Description:
 *      This function will return the 160-bit message digest into the
 *      Message_Digest array  provided by the caller.
 *      NOTE: The first octet of hash is stored in the 0th element,
 *            the last octet of hash in the 19th element.
 *
 *  Parameters:
 *      context: [in/out]
 *          The context to use to calculate the SHA-1 hash.
 *      Message_Digest: [out]
 *          Where the digest is returned.
 *
 *  Returns:
 *      sha Error Code.
 *
 */
int SHA1Result( SHA1Context *context,
                uint8_t Message_Digest[SHA1HashSize])
{
    int i;

    if (!context || !Message_Digest)
    {
        return shaNull;
    }

    if (context->Corrupted)
    {
        return context->Corrupted;
    }

    if (!context->Computed)
    {
        SHA1PadMessage(context);
        for(i=0; i<64; ++i)
        {
            /* message may be sensitive, clear it out */
            context->Message_Block[i] = 0;
        }
        context->Length_Low = 0;    /* and clear length */
        context->Length_High = 0;
        context->Computed = 1;
		 }

    for(i = 0; i < SHA1HashSize; ++i)
    {
        Message_Digest[i] = context->Intermediate_Hash[i>>2]
                            >> 8 * ( 3 - ( i & 0x03 ) );
    }

    return shaSuccess;
}

/*
 *  SHA1Input
 *
 *  Description:
 *      This function accepts an array of octets as the next portion
 *      of the message.
 *
 *  Parameters:
 *      context: [in/out]
 *          The SHA context to update
 *      message_array: [in]
 *          An array of characters representing the next portion of
 *          the message.
 *      length: [in]
 *          The length of the message in message_array
 *
 *  Returns:
 *      sha Error Code.
 *
 */
int SHA1Input(    SHA1Context    *context,
                  const void  *message_array_v,
                  unsigned       length)
{
	const uint8_t  *message_array = (const uint8_t*)message_array_v;
    if (!length)
    {
        return shaSuccess;
    }

    if (!context || !message_array)
    {
        return shaNull;
    }

    if (context->Computed)
    {
        context->Corrupted = shaStateError;
		return shaStateError;
    }

    if (context->Corrupted)
    {
         return context->Corrupted;
    }
    while(length-- && !context->Corrupted)
    {
    context->Message_Block[context->Message_Block_Index++] =
                    (*message_array & 0xFF);

    context->Length_Low += 8;
    if (context->Length_Low == 0)
    {
        context->Length_High++;
        if (context->Length_High == 0)
        {
            /* Message is too long */
            context->Corrupted = 1;
        }
    }

    if (context->Message_Block_Index == 64)
    {
        SHA1ProcessMessageBlock(context);
    }

    message_array++;
    }

    return shaSuccess;
}

/*
 *  SHA1ProcessMessageBlock
 *
 *  Description:
 *      This function will process the next 512 bits of the message
 *      stored in the Message_Block array.
 *
 *  Parameters:
 *      None.
 *
 *  Returns:
 *      Nothing.
 *
 *  Comments:
 *      Many of the variable names in this code, especially the
 *      single character names, were used because those were the
 *      names used in the publication.
 *
 *
 */
void SHA1ProcessMessageBlock(SHA1Context *context)
{
    const uint32_t K[] =    {       /* Constants defined in SHA-1   */
                            0x5A827999,
                            0x6ED9EBA1,
                            0x8F1BBCDC,
                            0xCA62C1D6
                            };
    int           t;                 /* Loop counter                */
    uint32_t      temp;              /* Temporary word value        */
    uint32_t      W[80];             /* Word sequence               */
    uint32_t      A, B, C, D, E;     /* Word buffers                */

    /*
     *  Initialize the first 16 words in the array W
     */
    for(t = 0; t < 16; t++)
    {
        W[t] = context->Message_Block[t * 4] << 24;
        W[t] |= context->Message_Block[t * 4 + 1] << 16;
        W[t] |= context->Message_Block[t * 4 + 2] << 8;
        W[t] |= context->Message_Block[t * 4 + 3];
    }

    for(t = 16; t < 80; t++)
    {
       W[t] = SHA1CircularShift(1,W[t-3] ^ W[t-8] ^ W[t-14] ^ W[t-16]);
    }

    A = context->Intermediate_Hash[0];
    B = context->Intermediate_Hash[1];
    C = context->Intermediate_Hash[2];
    D = context->Intermediate_Hash[3];
    E = context->Intermediate_Hash[4];

    for(t = 0; t < 20; t++)
    {
        temp =  SHA1CircularShift(5,A) +
                ((B & C) | ((~B) & D)) + E + W[t] + K[0];
        E = D;
        D = C;
        C = SHA1CircularShift(30,B);
		B = A;
        A = temp;
    }

    for(t = 20; t < 40; t++)
    {
        temp = SHA1CircularShift(5,A) + (B ^ C ^ D) + E + W[t] + K[1];
        E = D;
        D = C;
        C = SHA1CircularShift(30,B);
        B = A;
        A = temp;
    }

    for(t = 40; t < 60; t++)
    {
        temp = SHA1CircularShift(5,A) +
               ((B & C) | (B & D) | (C & D)) + E + W[t] + K[2];
        E = D;
        D = C;
        C = SHA1CircularShift(30,B);
        B = A;
        A = temp;
    }

    for(t = 60; t < 80; t++)
    {
        temp = SHA1CircularShift(5,A) + (B ^ C ^ D) + E + W[t] + K[3];
        E = D;
        D = C;
        C = SHA1CircularShift(30,B);
        B = A;
        A = temp;
    }

    context->Intermediate_Hash[0] += A;
    context->Intermediate_Hash[1] += B;
    context->Intermediate_Hash[2] += C;
    context->Intermediate_Hash[3] += D;
    context->Intermediate_Hash[4] += E;

    context->Message_Block_Index = 0;
}


/*
 *  SHA1PadMessage
 *
 *  Description:
 *      According to the standard, the message must be padded to an even
 *      512 bits.  The first padding bit must be a '1'.  The last 64
 *      bits represent the length of the original message.  All bits in
 *      between should be 0.  This function will pad the message
 *      according to those rules by filling the Message_Block array
 *      accordingly.  It will also call the ProcessMessageBlock function
 *      provided appropriately.  When it returns, it can be assumed that
 *      the message digest has been computed.
 *
 *  Parameters:
 *      context: [in/out]
 *          The context to pad
 *      ProcessMessageBlock: [in]
 *          The appropriate SHA*ProcessMessageBlock function
 *  Returns:
 *      Nothing.
 *
 */

void SHA1PadMessage(SHA1Context *context)
{
    /*
     *  Check to see if the current message block is too small to hold
     *  the initial padding bits and length.  If so, we will pad the
     *  block, process it, and then continue padding into a second
     *  block.
     */
    if (context->Message_Block_Index > 55)
    {
        context->Message_Block[context->Message_Block_Index++] = 0x80;
        while(context->Message_Block_Index < 64)
        {
            context->Message_Block[context->Message_Block_Index++] = 0;
        }

        SHA1ProcessMessageBlock(context);

        while(context->Message_Block_Index < 56)
        {
            context->Message_Block[context->Message_Block_Index++] = 0;
        }
    }
    else
    {
        context->Message_Block[context->Message_Block_Index++] = 0x80;
        while(context->Message_Block_Index < 56)
        {
			context->Message_Block[context->Message_Block_Index++] = 0;
        }
    }

    /*
     *  Store the message length as the last 8 octets
     */
    context->Message_Block[56] = context->Length_High >> 24;
    context->Message_Block[57] = context->Length_High >> 16;
    context->Message_Block[58] = context->Length_High >> 8;
    context->Message_Block[59] = context->Length_High;
    context->Message_Block[60] = context->Length_Low >> 24;
    context->Message_Block[61] = context->Length_Low >> 16;
    context->Message_Block[62] = context->Length_Low >> 8;
    context->Message_Block[63] = context->Length_Low;

    SHA1ProcessMessageBlock(context);
}
